part of '../flutter_compute_tree.dart';

abstract class Trigger<T> {
  void call();
  void withArg(T arg);

  T get payload;
  Future<void> get called;

  Result<K> asyncHandler<K, R extends Object>(
    Future<K> Function() fn, {
    void Function()? abort,
  });

  K handler<K>(K Function() fn, K initialValue);
}

class _TriggerUnit<T> extends Unit<Trigger<T>, Object> implements Trigger<T> {
  var _value = Object();
  late T _arg;
  final _completer = Completer<void>();

  @override
  T get payload => _arg;

  @override
  Future<void> get called => _completer.future;

  _TriggerUnit();

  @override
  Trigger<T> unwrap() {
    return this;
  }

  @override
  void call() {
    if (!_completer.isCompleted) {
      _completer.complete();
    }

    _value = Object();
    notify();
  }

  @override
  void withArg(T arg) {
    _arg = arg;
    call();
  }

  @override
  Result<K> asyncHandler<K, R extends Object>(
    Future<K> Function() fn, {
    void Function()? abort,
  }) {
    return invokeAsync(
      fn,
      abort: abort,
      deferredUntil: called,
      dep: _value,
    );
  }

  @override
  K handler<K>(K Function() fn, K initialValue) {
    return invoke.deferred<K, Object>(
      fn,
      until: called,
      initialValue: initialValue,
      dep: _value,
    );
  }

  @override
  int get hashCode => _value.hashCode;

  @override
  bool operator ==(Object other) {
    return other is _TriggerUnit<T> && other._value == _value;
  }
}
